@@ -20,10 +20,10 @@ set -e
 usage() {
     echo
     echo "Usage: $(basename "$0")"
-    echo "            [-a i386 | **amd64** | armhf | aarch64 | ppc64le]"
+    echo "            [-a **amd64** | armhf | aarch64 | ppc64le]"
     echo "            [-b **haproxy** ]"
     echo "            [-c **~/.cache/image-create** | <cache directory> ]"
-    echo "            [-d **focal**/**8** | <other release id> ]"
+    echo "            [-d **jammy**/**9-stream**/**9** | <other release id> ]"
     echo "            [-e]"
     echo "            [-f]"
     echo "            [-g **repository branch** | stable/train | stable/stein | ... ]"
@@ -31,8 +31,9 @@ usage() {
     echo "            [-i **ubuntu-minimal** | fedora | centos-minimal | rhel ]"
     echo "            [-k <kernel package name> ]"
     echo "            [-l <log file> ]"
+    echo "            [-m]"
     echo "            [-n]"
-    echo "            [-o **amphora-x64-haproxy** | <filename> ]"
+    echo "            [-o **amphora-x64-haproxy.qcow2** | <filename> ]"
     echo "            [-p]"
     echo "            [-r <root password> ]"
     echo "            [-s **2** | <size in GB> ]"
@@ -44,7 +45,7 @@ usage() {
     echo "        '-a' is the architecture type for the image (default: amd64)"
     echo "        '-b' is the backend type (default: haproxy)"
     echo "        '-c' is the path to the cache directory (default: ~/.cache/image-create)"
-    echo "        '-d' distribution release id (default on ubuntu: focal)"
+    echo "        '-d' distribution release id (default on ubuntu: jammy)"
     echo "        '-e' enable complete mandatory access control systems when available (default: permissive)"
     echo "        '-f' disable tmpfs for build"
     echo "        '-g' build the image for a specific OpenStack Git branch (default: current repository branch)"
@@ -52,6 +53,7 @@ usage() {
     echo "        '-i' is the base OS (default: ubuntu-minimal)"
     echo "        '-k' is the kernel meta package name, currently only for ubuntu-minimal base OS (default: linux-image-virtual)"
     echo "        '-l' is output logfile (default: none)"
+    echo "        '-m' enable vCPU pinning optimizations (default: disabled)"
     echo "        '-n' disable sshd (default: enabled)"
     echo "        '-o' is the output image file name"
     echo "        '-p' install amphora-agent from distribution packages (default: disabled)"
@@ -92,12 +94,11 @@ dib_enable_tracing=
 
 AMP_LOGFILE=""
 
-while getopts "a:b:c:d:efg:hi:k:l:no:pt:r:s:vw:xy" opt; do
+while getopts "a:b:c:d:efg:hi:k:l:mno:pt:r:s:vw:xy" opt; do
     case $opt in
         a)
             AMP_ARCH=$OPTARG
-            if [ "$AMP_ARCH" != "i386" ] && \
-                [ "$AMP_ARCH" != "amd64" ] && \
+            if [ "$AMP_ARCH" != "amd64" ] && \
                 [ "$AMP_ARCH" != "ppc64le" ] && \
                 [ "$AMP_ARCH" != "aarch64" ] && \
                 [ "$AMP_ARCH" != "armhf" ]; then
@@ -166,6 +167,9 @@ while getopts "a:b:c:d:efg:hi:k:l:no:pt:r:s:vw:xy" opt; do
         l)
             AMP_LOGFILE="--logfile=$OPTARG"
         ;;
+        m)
+            AMP_ENABLE_CPUPINNING=1
+        ;;
         n)
             AMP_DISABLE_SSHD=1
         ;;
@@ -234,14 +238,16 @@ AMP_CACHEDIR="$( cd "$AMP_CACHEDIR" && pwd )"
 AMP_BASEOS=${AMP_BASEOS:-"ubuntu-minimal"}
 
 if [ "$AMP_BASEOS" = "ubuntu-minimal" ]; then
-    export DIB_RELEASE=${AMP_DIB_RELEASE:-"focal"}
-elif [ "${AMP_BASEOS}" = "centos-minimal" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
-    export DIB_RELEASE=${AMP_DIB_RELEASE:-"8"}
+    export DIB_RELEASE=${AMP_DIB_RELEASE:-"jammy"}
+elif [ "${AMP_BASEOS}" = "rhel" ]; then
+    export DIB_RELEASE=${AMP_DIB_RELEASE:-"9"}
+elif [ "${AMP_BASEOS}" = "centos-minimal" ]; then
+    export DIB_RELEASE=${AMP_DIB_RELEASE:-"9-stream"}
 elif [ "${AMP_BASEOS}" = "fedora" ]; then
     export DIB_RELEASE=${AMP_DIB_RELEASE:-"28"}
 fi
 
-AMP_OUTPUTFILENAME=${AMP_OUTPUTFILENAME:-"$PWD/amphora-x64-haproxy"}
+AMP_OUTPUTFILENAME=${AMP_OUTPUTFILENAME:-"$PWD/amphora-x64-haproxy.qcow2"}
 
 AMP_IMAGETYPE=${AMP_IMAGETYPE:-"qcow2"}
 
@@ -251,6 +257,8 @@ if [ "$AMP_BASEOS" = "ubuntu-minimal" ]; then
     export DIB_UBUNTU_KERNEL=${AMP_KERNEL:-"linux-image-virtual"}
 fi
 
+AMP_ENABLE_CPUPINNING=${AMP_ENABLE_CPUPINNING:-0}
+
 AMP_DISABLE_SSHD=${AMP_DISABLE_SSHD:-0}
 
 AMP_PACKAGE_INSTALL=${AMP_PACKAGE_INSTALL:-0}
@@ -474,6 +482,11 @@ AMP_element_sequence="$AMP_element_sequence pip-cache"
 # Add certificate ramfs element
 AMP_element_sequence="$AMP_element_sequence certs-ramfs"
 
+# Add cpu-pinning element
+if [ "$AMP_ENABLE_CPUPINNING" -eq 1 ]; then
+    AMP_element_sequence="$AMP_element_sequence cpu-pinning"
+fi
+
 # Disable SSHD if requested
 if [ "$AMP_DISABLE_SSHD" -eq 1 ]; then
     AMP_element_sequence="$AMP_element_sequence remove-sshd"
@@ -521,4 +534,4 @@ if [ -z "$DIB_REPOREF_amphora_agent" ]; then
 else
     echo "Successfully built the amphora using the $DIB_REPOREF_amphora_agent amphora-agent."
 fi
-echo "Amphora image size: `stat -c "%n %s" $AMP_OUTPUTFILENAME.$AMP_IMAGETYPE`"
+echo "Amphora image size: `stat -c "%n %s" $AMP_OUTPUTFILENAME`"
