@@ -14,9 +14,6 @@
 # under the License.
 
 import abc
-from typing import Optional
-
-from octavia.db import models as db_models
 
 
 class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
@@ -154,7 +151,7 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         """
 
     def post_vip_plug(self, amphora, load_balancer, amphorae_network_config,
-                      vrrp_port=None, vip_subnet=None):
+                      vrrp_port, vip_subnet, additional_vip_data=None):
         """Called after network driver has allocated and plugged the VIP
 
         :param amphora:
@@ -174,8 +171,13 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         :type vip_subnet: octavia.network.data_models.Subnet
 
         :type vip_network: octavia.network.data_models.AmphoraNetworkConfig
+
+        :type additional_vip_data: list of
+                                   octavia.network.data_models.AdditionalVipData
+
         :returns: None
 
+
         This is to do any additional work needed on the amphorae to plug
         the vip, such as bring up interfaces.
         """
@@ -234,19 +236,6 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         :type timeout_dict: dict
         """
 
-    @abc.abstractmethod
-    def check(self, amphora: db_models.Amphora,
-              timeout_dict: Optional[dict] = None):
-        """Check connectivity to the amphora.
-
-        :param amphora: The amphora to query.
-        :param timeout_dict: Dictionary of timeout values for calls to the
-                             amphora. May contain: req_conn_timeout,
-                             req_read_timeout, conn_max_retries,
-                             conn_retry_interval
-        :raises TimeOutException: The amphora didn't reply
-        """
-
 
 class VRRPDriverMixin(object, metaclass=abc.ABCMeta):
     """Abstract mixin class for VRRP support in loadbalancer amphorae
