@@ -220,7 +220,7 @@ class PaginationHelper(object):
 
     @staticmethod
     def _prepare_tags_list(param):
-        """Split comma seperated tags and return a flat list of tags."""
+        """Split comma separated tags and return a flat list of tags."""
         if not isinstance(param, list):
             param = [param]
         return list(itertools.chain.from_iterable(
@@ -359,16 +359,16 @@ class PaginationHelper(object):
                         default = PaginationHelper._get_default_column_value(
                             model_attr.property.columns[0].type)
                         attr = sa_sql.expression.case(
-                            [(model_attr.isnot(None),
-                              model_attr), ], else_=default)
+                            (model_attr.isnot(None), model_attr),
+                            else_=default)
                         crit_attrs.append((attr == marker_values[j]))
 
                     model_attr = getattr(model, self.sort_keys[i][0])
                     default = PaginationHelper._get_default_column_value(
                         model_attr.property.columns[0].type)
                     attr = sa_sql.expression.case(
-                        [(model_attr.isnot(None),
-                          model_attr), ], else_=default)
+                        (model_attr.isnot(None), model_attr),
+                        else_=default)
                     this_sort_dir = self.sort_keys[i][1]
                     if this_sort_dir == constants.DESC:
                         if self.page_reverse == "True":
