@@ -38,7 +38,6 @@ from octavia.common import exceptions
 from octavia.common import stats
 from octavia.common import utils
 from octavia.common import validate
-from octavia.db import api as db_api
 from octavia.db import prepare as db_prepare
 from octavia.i18n import _
 from octavia.network import base as network_base
@@ -59,8 +58,9 @@ class LoadBalancersController(base.BaseController):
     def get_one(self, id, fields=None):
         """Gets a single load balancer's details."""
         context = pecan_request.context.get('octavia_context')
-        load_balancer = self._get_db_lb(context.session, id,
-                                        show_deleted=False)
+        with context.session.begin():
+            load_balancer = self._get_db_lb(context.session, id,
+                                            show_deleted=False)
 
         if not load_balancer:
             raise exceptions.NotFound(
@@ -85,11 +85,13 @@ class LoadBalancersController(base.BaseController):
 
         query_filter = self._auth_get_all(context, project_id)
 
-        load_balancers, links = (
-            self.repositories.load_balancer.get_all_API_list(
-                context.session, show_deleted=False,
-                pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
-                **query_filter))
+        with context.session.begin():
+            load_balancers, links = (
+                self.repositories.load_balancer.get_all_API_list(
+                    context.session, show_deleted=False,
+                    pagination_helper=pcontext.get(
+                        constants.PAGINATION_HELPER),
+                    **query_filter))
         result = self._convert_db_to_type(
             load_balancers, [lb_types.LoadBalancerResponse])
         if fields is not None:
@@ -223,6 +225,31 @@ class LoadBalancersController(base.BaseController):
                 "VIP port's subnet could not be determined. Please "
                 "specify either a VIP subnet or address."))
 
+    @staticmethod
+    def _validate_subnets_share_network_but_no_duplicates(load_balancer):
+        # Validate that no subnet_id is used more than once
+        subnet_use_counts = {load_balancer.vip_subnet_id: 1}
+        for vip in load_balancer.additional_vips:
+            if vip.subnet_id in subnet_use_counts:
+                raise exceptions.ValidationException(detail=_(
+                    'Duplicate VIP subnet(s) specified. Only one IP can be '
+                    'bound per subnet.'))
+            subnet_use_counts[vip.subnet_id] = 1
+
+        # Validate that all subnets belong to the same network
+        network_driver = utils.get_network_driver()
+        used_subnets = {}
+        for subnet_id in subnet_use_counts:
+            used_subnets[subnet_id] = network_driver.get_subnet(subnet_id)
+        all_networks = [subnet.network_id for subnet in used_subnets.values()]
+        if len(set(all_networks)) > 1:
+            raise exceptions.ValidationException(detail=_(
+                'All VIP subnets must belong to the same network.'
+            ))
+        # Fill the network_id for each additional_vip
+        for vip in load_balancer.additional_vips:
+            vip.network_id = used_subnets[vip.subnet_id].network_id
+
     def _validate_vip_request_object(self, load_balancer, context=None):
         allowed_network_objects = []
         if CONF.networking.allow_vip_port_id:
@@ -270,7 +297,18 @@ class LoadBalancersController(base.BaseController):
             validate.qos_policy_exists(
                 qos_policy_id=load_balancer.vip_qos_policy_id)
 
-    def _create_vip_port_if_not_exist(self, load_balancer_db):
+        # Even though we've just validated the subnet or else retrieved its ID
+        # directly from the port, we might still be missing the network.
+        if not load_balancer.vip_network_id:
+            subnet = validate.subnet_exists(
+                subnet_id=load_balancer.vip_subnet_id)
+            load_balancer.vip_network_id = subnet.network_id
+
+        # Multi-vip validation for ensuring subnets are "sane"
+        self._validate_subnets_share_network_but_no_duplicates(load_balancer)
+
+    @staticmethod
+    def _create_vip_port_if_not_exist(load_balancer_db):
         """Create vip port."""
         network_driver = utils.get_network_driver()
         try:
@@ -291,8 +329,9 @@ class LoadBalancersController(base.BaseController):
         provider = None
         if not isinstance(load_balancer.flavor_id, wtypes.UnsetType):
             try:
-                provider = self.repositories.flavor.get_flavor_provider(
-                    session, load_balancer.flavor_id)
+                with session.begin():
+                    provider = self.repositories.flavor.get_flavor_provider(
+                        session, load_balancer.flavor_id)
             except sa_exception.NoResultFound as e:
                 raise exceptions.ValidationException(
                     detail=_("Invalid flavor_id.")) from e
@@ -341,8 +380,9 @@ class LoadBalancersController(base.BaseController):
 
     def _validate_flavor(self, session, load_balancer):
         if not isinstance(load_balancer.flavor_id, wtypes.UnsetType):
-            flavor = self.repositories.flavor.get(session,
-                                                  id=load_balancer.flavor_id)
+            with session.begin():
+                flavor = self.repositories.flavor.get(
+                    session, id=load_balancer.flavor_id)
             if not flavor:
                 raise exceptions.ValidationException(
                     detail=_("Invalid flavor_id."))
@@ -380,8 +420,9 @@ class LoadBalancersController(base.BaseController):
 
     def _validate_availability_zone(self, session, load_balancer):
         if not isinstance(load_balancer.availability_zone, wtypes.UnsetType):
-            az = self.repositories.availability_zone.get(
-                session, name=load_balancer.availability_zone)
+            with session.begin():
+                az = self.repositories.availability_zone.get(
+                    session, name=load_balancer.availability_zone)
             if not az:
                 raise exceptions.ValidationException(
                     detail=_("Invalid availability zone."))
@@ -420,7 +461,8 @@ class LoadBalancersController(base.BaseController):
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
-        lock_session = db_api.get_session(autocommit=False)
+        lock_session = context.session
+        lock_session.begin()
         try:
             if self.repositories.check_quota_met(
                     context.session,
@@ -436,6 +478,7 @@ class LoadBalancersController(base.BaseController):
                 render_unsets=False
             ))
             vip_dict = lb_dict.pop('vip', {})
+            additional_vip_dicts = lb_dict.pop('additional_vips', [])
 
             # Make sure we store the right provider in the DB
             lb_dict['provider'] = driver.name
@@ -455,7 +498,7 @@ class LoadBalancersController(base.BaseController):
                 valid_networks=az_dict.get(constants.VALID_VIP_NETWORKS))
 
             db_lb = self.repositories.create_load_balancer_and_vip(
-                lock_session, lb_dict, vip_dict)
+                lock_session, lb_dict, vip_dict, additional_vip_dicts)
 
             # Pass the flavor dictionary through for the provider drivers
             # This is a "virtual" lb_dict item that includes the expanded
@@ -473,14 +516,20 @@ class LoadBalancersController(base.BaseController):
             try:
                 provider_vip_dict = driver_utils.vip_dict_to_provider_dict(
                     vip_dict)
-                vip_dict = driver_utils.call_provider(
+                provider_additional_vips = [
+                    driver_utils.additional_vip_dict_to_provider_dict(add_vip)
+                    for add_vip in additional_vip_dicts]
+                vip_dict, additional_vip_dicts = driver_utils.call_provider(
                     driver.name, driver.create_vip_port, db_lb.id,
-                    db_lb.project_id, provider_vip_dict)
+                    db_lb.project_id, provider_vip_dict,
+                    provider_additional_vips)
                 vip = driver_utils.provider_vip_dict_to_vip_obj(vip_dict)
+                add_vips = [data_models.AdditionalVip(**add_vip)
+                            for add_vip in additional_vip_dicts]
             except exceptions.ProviderNotImplementedError:
                 # create vip port if not exist, driver didn't want to create
                 # the VIP port
-                vip = self._create_vip_port_if_not_exist(db_lb)
+                vip, add_vips = self._create_vip_port_if_not_exist(db_lb)
                 LOG.info('Created VIP port %s for provider %s.',
                          vip.port_id, driver.name)
                 # If a port_id wasn't passed in and we made it this far
@@ -496,6 +545,11 @@ class LoadBalancersController(base.BaseController):
                 lock_session, db_lb.id, ip_address=vip.ip_address,
                 port_id=vip.port_id, network_id=vip.network_id,
                 subnet_id=vip.subnet_id, octavia_owned=octavia_owned)
+            for add_vip in add_vips:
+                self.repositories.additional_vip.update(
+                    lock_session, db_lb.id, ip_address=add_vip.ip_address,
+                    port_id=add_vip.port_id, network_id=add_vip.network_id,
+                    subnet_id=add_vip.subnet_id)
 
             if listeners or pools:
                 db_pools, db_lists = self._graph_create(
@@ -503,7 +557,9 @@ class LoadBalancersController(base.BaseController):
 
             # Prepare the data for the driver data model
             driver_lb_dict = driver_utils.lb_dict_to_provider_dict(
-                lb_dict, vip, db_pools, db_lists)
+                lb_dict, vip, add_vips, db_pools, db_lists)
+
+            lock_session.flush()
 
             # Dispatch to the driver
             LOG.info("Sending create Load Balancer %s to provider %s",
@@ -520,7 +576,8 @@ class LoadBalancersController(base.BaseController):
             with excutils.save_and_reraise_exception():
                 lock_session.rollback()
 
-        db_lb = self._get_db_lb(context.session, db_lb.id)
+        with context.session.begin():
+            db_lb = self._get_db_lb(context.session, db_lb.id)
 
         result = self._convert_db_to_type(
             db_lb, lb_types.LoadBalancerFullResponse)
@@ -641,7 +698,8 @@ class LoadBalancersController(base.BaseController):
         """Updates a load balancer."""
         load_balancer = load_balancer.loadbalancer
         context = pecan_request.context.get('octavia_context')
-        db_lb = self._get_db_lb(context.session, id, show_deleted=False)
+        with context.session.begin():
+            db_lb = self._get_db_lb(context.session, id, show_deleted=False)
 
         self._auth_validate_action(context, db_lb.project_id,
                                    constants.RBAC_PUT)
@@ -656,8 +714,8 @@ class LoadBalancersController(base.BaseController):
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(db_lb.provider)
 
-        with db_api.get_lock_session() as lock_session:
-            self._test_lb_status(lock_session, id)
+        with context.session.begin():
+            self._test_lb_status(context.session, id)
 
             # Prepare the data for the driver data model
             lb_dict = load_balancer.to_dict(render_unsets=False)
@@ -683,15 +741,17 @@ class LoadBalancersController(base.BaseController):
             db_lb_dict = load_balancer.to_dict(render_unsets=False)
             if 'vip' in db_lb_dict:
                 db_vip_dict = db_lb_dict.pop('vip')
-                self.repositories.vip.update(lock_session, id, **db_vip_dict)
+                self.repositories.vip.update(context.session, id,
+                                             **db_vip_dict)
             if db_lb_dict:
-                self.repositories.load_balancer.update(lock_session, id,
+                self.repositories.load_balancer.update(context.session, id,
                                                        **db_lb_dict)
 
         # Force SQL alchemy to query the DB, otherwise we get inconsistent
         # results
         context.session.expire_all()
-        db_lb = self._get_db_lb(context.session, id)
+        with context.session.begin():
+            db_lb = self._get_db_lb(context.session, id)
         result = self._convert_db_to_type(db_lb, lb_types.LoadBalancerResponse)
         return lb_types.LoadBalancerRootResponse(loadbalancer=result)
 
@@ -700,7 +760,8 @@ class LoadBalancersController(base.BaseController):
         """Deletes a load balancer."""
         context = pecan_request.context.get('octavia_context')
         cascade = strutils.bool_from_string(cascade)
-        db_lb = self._get_db_lb(context.session, id, show_deleted=False)
+        with context.session.begin():
+            db_lb = self._get_db_lb(context.session, id, show_deleted=False)
 
         self._auth_validate_action(context, db_lb.project_id,
                                    constants.RBAC_DELETE)
@@ -708,13 +769,13 @@ class LoadBalancersController(base.BaseController):
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(db_lb.provider)
 
-        with db_api.get_lock_session() as lock_session:
+        with context.session.begin():
             if (db_lb.listeners or db_lb.pools) and not cascade:
                 msg = _("Cannot delete Load Balancer %s - "
                         "it has children") % id
                 LOG.warning(msg)
                 raise exceptions.ValidationException(detail=msg)
-            self._test_lb_status(lock_session, id,
+            self._test_lb_status(context.session, id,
                                  lb_status=constants.PENDING_DELETE)
 
             LOG.info("Sending delete Load Balancer %s to provider %s",
@@ -765,8 +826,9 @@ class StatusController(base.BaseController):
                          status_code=200)
     def get(self):
         context = pecan_request.context.get('octavia_context')
-        load_balancer = self._get_db_lb(context.session, self.id,
-                                        show_deleted=False)
+        with context.session.begin():
+            load_balancer = self._get_db_lb(context.session, self.id,
+                                            show_deleted=False)
         if not load_balancer:
             LOG.info("Load balancer %s not found.", id)
             raise exceptions.NotFound(
@@ -793,8 +855,9 @@ class StatisticsController(base.BaseController, stats.StatsMixin):
                          status_code=200)
     def get(self):
         context = pecan_request.context.get('octavia_context')
-        load_balancer = self._get_db_lb(context.session, self.id,
-                                        show_deleted=False)
+        with context.session.begin():
+            load_balancer = self._get_db_lb(context.session, self.id,
+                                            show_deleted=False)
         if not load_balancer:
             LOG.info("Load balancer %s not found.", id)
             raise exceptions.NotFound(
@@ -804,7 +867,8 @@ class StatisticsController(base.BaseController, stats.StatsMixin):
         self._auth_validate_action(context, load_balancer.project_id,
                                    constants.RBAC_GET_STATS)
 
-        lb_stats = self.get_loadbalancer_stats(context.session, self.id)
+        with context.session.begin():
+            lb_stats = self.get_loadbalancer_stats(context.session, self.id)
 
         result = self._convert_db_to_type(
             lb_stats, lb_types.LoadBalancerStatisticsResponse)
@@ -821,8 +885,9 @@ class FailoverController(LoadBalancersController):
     def put(self, **kwargs):
         """Fails over a loadbalancer"""
         context = pecan_request.context.get('octavia_context')
-        db_lb = self._get_db_lb(context.session, self.lb_id,
-                                show_deleted=False)
+        with context.session.begin():
+            db_lb = self._get_db_lb(context.session, self.lb_id,
+                                    show_deleted=False)
 
         self._auth_validate_action(context, db_lb.project_id,
                                    constants.RBAC_PUT_FAILOVER)
@@ -830,8 +895,9 @@ class FailoverController(LoadBalancersController):
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(db_lb.provider)
 
-        with db_api.get_lock_session() as lock_session:
-            self._test_and_set_failover_prov_status(lock_session, self.lb_id)
+        with context.session.begin():
+            self._test_and_set_failover_prov_status(context.session,
+                                                    self.lb_id)
             LOG.info("Sending failover request for load balancer %s to the "
                      "provider %s", self.lb_id, driver.name)
             driver_utils.call_provider(
