@@ -78,6 +78,8 @@ def _mutate_config(*args, **kwargs):
 
 def main():
     service.prepare_service(sys.argv)
+    LOG.debug('Full set of CONF:')
+    CONF.log_opt_values(LOG, logging.DEBUG)
 
     gmr.TextGuruMeditation.setup_autorun(version)
 
