@@ -16,6 +16,7 @@
 
 import datetime
 import re
+import typing as tp
 
 from oslo_log import log as logging
 from sqlalchemy.orm import collections
@@ -419,7 +420,8 @@ class Listener(BaseDataModel):
                  tags=None, client_ca_tls_certificate_id=None,
                  client_authentication=None, client_crl_container_id=None,
                  allowed_cidrs=None, tls_ciphers=None, tls_versions=None,
-                 alpn_protocols=None):
+                 alpn_protocols=None, hsts_max_age=None,
+                 hsts_include_subdomains=None, hsts_preload=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -455,6 +457,10 @@ class Listener(BaseDataModel):
         self.tls_ciphers = tls_ciphers
         self.tls_versions = tls_versions
         self.alpn_protocols = alpn_protocols
+        self.hsts_max_age: tp.Optional[int] = hsts_max_age
+        self.hsts_include_subdomains: tp.Optional[bool] = (
+            hsts_include_subdomains)
+        self.hsts_preload: tp.Optional[bool] = hsts_preload
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -496,7 +502,8 @@ class LoadBalancer(BaseDataModel):
                  topology=None, vip=None, listeners=None, amphorae=None,
                  pools=None, vrrp_group=None, server_group_id=None,
                  created_at=None, updated_at=None, provider=None, tags=None,
-                 flavor_id=None, availability_zone=None):
+                 flavor_id=None, availability_zone=None,
+                 additional_vips=None):
 
         self.id = id
         self.project_id = project_id
@@ -518,6 +525,7 @@ class LoadBalancer(BaseDataModel):
         self.tags = tags or []
         self.flavor_id = flavor_id
         self.availability_zone = availability_zone
+        self.additional_vips = additional_vips or []
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -560,6 +568,18 @@ class Vip(BaseDataModel):
         self.octavia_owned = octavia_owned
 
 
+class AdditionalVip(BaseDataModel):
+
+    def __init__(self, load_balancer_id=None, ip_address=None, subnet_id=None,
+                 network_id=None, port_id=None, load_balancer=None):
+        self.load_balancer_id = load_balancer_id
+        self.ip_address = ip_address
+        self.subnet_id = subnet_id
+        self.network_id = network_id
+        self.port_id = port_id
+        self.load_balancer = load_balancer
+
+
 class SNI(BaseDataModel):
 
     def __init__(self, listener_id=None, position=None, listener=None,
