@@ -31,6 +31,7 @@ from octavia.controller.worker.v2.tasks import compute_tasks
 from octavia.controller.worker.v2.tasks import database_tasks
 from octavia.controller.worker.v2.tasks import lifecycle_tasks
 from octavia.controller.worker.v2.tasks import network_tasks
+from octavia.controller.worker.v2.tasks import notification_tasks
 from octavia.db import repositories as repo
 
 CONF = cfg.CONF
@@ -66,10 +67,13 @@ class LoadBalancerFlows(object):
         ))
         lb_create_flow.add(network_tasks.AllocateVIP(
             requires=constants.LOADBALANCER,
-            provides=constants.VIP))
+            provides=(constants.VIP, constants.ADDITIONAL_VIPS)))
         lb_create_flow.add(database_tasks.UpdateVIPAfterAllocation(
             requires=(constants.LOADBALANCER_ID, constants.VIP),
             provides=constants.LOADBALANCER))
+        lb_create_flow.add(database_tasks.UpdateAdditionalVIPsAfterAllocation(
+            requires=(constants.LOADBALANCER_ID, constants.ADDITIONAL_VIPS),
+            provides=constants.LOADBALANCER))
         lb_create_flow.add(network_tasks.UpdateVIPSecurityGroup(
             requires=constants.LOADBALANCER_ID))
         lb_create_flow.add(network_tasks.GetSubnetFromVIP(
@@ -99,6 +103,13 @@ class LoadBalancerFlows(object):
             )
         )
 
+        if CONF.controller_worker.event_notifications:
+            lb_create_flow.add(
+                notification_tasks.SendCreateNotification(
+                    requires=constants.LOADBALANCER
+                )
+            )
+
         return lb_create_flow
 
     def _create_single_topology(self):
@@ -311,6 +322,9 @@ class LoadBalancerFlows(object):
             requires=constants.LOADBALANCER))
         delete_LB_flow.add(database_tasks.DecrementLoadBalancerQuota(
             requires=constants.PROJECT_ID))
+        if CONF.controller_worker.event_notifications:
+            delete_LB_flow.add(notification_tasks.SendDeleteNotification(
+                requires=constants.LOADBALANCER))
         return delete_LB_flow
 
     def get_cascade_delete_load_balancer_flow(self, lb, listeners, pools):
@@ -338,6 +352,12 @@ class LoadBalancerFlows(object):
             requires=[constants.LOADBALANCER, constants.UPDATE_DICT]))
         update_LB_flow.add(database_tasks.MarkLBActiveInDB(
             requires=constants.LOADBALANCER))
+        if CONF.controller_worker.event_notifications:
+            update_LB_flow.add(
+                notification_tasks.SendUpdateNotification(
+                    requires=constants.LOADBALANCER
+                )
+            )
 
         return update_LB_flow
 
@@ -406,12 +426,18 @@ class LoadBalancerFlows(object):
         # Check that the VIP port exists and is ok
         failover_LB_flow.add(
             network_tasks.AllocateVIPforFailover(
-                requires=constants.LOADBALANCER, provides=constants.VIP))
+                requires=constants.LOADBALANCER,
+                provides=(constants.VIP, constants.ADDITIONAL_VIPS)))
 
         # Update the database with the VIP information
         failover_LB_flow.add(database_tasks.UpdateVIPAfterAllocation(
             requires=(constants.LOADBALANCER_ID, constants.VIP),
             provides=constants.LOADBALANCER))
+        failover_LB_flow.add(
+            database_tasks.UpdateAdditionalVIPsAfterAllocation(
+                requires=(constants.LOADBALANCER_ID,
+                          constants.ADDITIONAL_VIPS),
+                provides=constants.LOADBALANCER))
 
         # Make sure the SG has the correct rules and re-apply to the
         # VIP port. It is not used on the VIP port, but will help lock
@@ -611,14 +637,6 @@ class LoadBalancerFlows(object):
                 requires=constants.LOADBALANCER_ID,
                 provides=constants.AMPHORAE))
 
-            failover_LB_flow.add(
-                amphora_driver_tasks.AmphoraeGetConnectivityStatus(
-                    name=(new_amp_role + '-' +
-                          constants.AMPHORAE_GET_CONNECTIVITY_STATUS),
-                    requires=constants.AMPHORAE,
-                    rebind={constants.NEW_AMPHORA_ID: constants.AMPHORA_ID},
-                    provides=constants.AMPHORAE_STATUS))
-
             # Listeners update needs to be run on all amphora to update
             # their peer configurations. So parallelize this with an
             # unordered subflow.
@@ -633,18 +651,14 @@ class LoadBalancerFlows(object):
                 amphora_driver_tasks.AmphoraIndexListenerUpdate(
                     name=(constants.AMPHORA + '-0-' +
                           constants.AMP_LISTENER_UPDATE),
-                    requires=(constants.LOADBALANCER, constants.AMPHORAE,
-                              constants.AMPHORAE_STATUS),
-                    rebind={constants.NEW_AMPHORA_ID: constants.AMPHORA_ID},
+                    requires=(constants.LOADBALANCER, constants.AMPHORAE),
                     inject={constants.AMPHORA_INDEX: 0,
                             constants.TIMEOUT_DICT: timeout_dict}))
             update_amps_subflow.add(
                 amphora_driver_tasks.AmphoraIndexListenerUpdate(
                     name=(constants.AMPHORA + '-1-' +
                           constants.AMP_LISTENER_UPDATE),
-                    requires=(constants.LOADBALANCER, constants.AMPHORAE,
-                              constants.AMPHORAE_STATUS),
-                    rebind={constants.NEW_AMPHORA_ID: constants.AMPHORA_ID},
+                    requires=(constants.LOADBALANCER, constants.AMPHORAE),
                     inject={constants.AMPHORA_INDEX: 1,
                             constants.TIMEOUT_DICT: timeout_dict}))
 
@@ -653,8 +667,7 @@ class LoadBalancerFlows(object):
             # Configure and enable keepalived in the amphora
             failover_LB_flow.add(self.amp_flows.get_vrrp_subflow(
                 new_amp_role + '-' + constants.GET_VRRP_SUBFLOW,
-                timeout_dict, create_vrrp_group=False,
-                get_amphorae_status=False))
+                timeout_dict, create_vrrp_group=False))
 
             # #### End of standby ####
 
@@ -669,7 +682,6 @@ class LoadBalancerFlows(object):
                     name=(new_amp_role + '-' +
                           constants.AMPHORA_RELOAD_LISTENER),
                     requires=(constants.LOADBALANCER, constants.AMPHORAE),
-                    rebind={constants.NEW_AMPHORA_ID: constants.AMPHORA_ID},
                     inject={constants.AMPHORA_INDEX: 1,
                             constants.TIMEOUT_DICT: timeout_dict}))
 
