@@ -19,6 +19,7 @@ from oslo_db.sqlalchemy import models
 import sqlalchemy as sa
 from sqlalchemy.ext import orderinglist
 from sqlalchemy import orm
+from sqlalchemy.orm import Mapped
 from sqlalchemy.orm import validates
 from sqlalchemy.sql import func
 from sqlalchemy_utils import ScalarListType
@@ -353,7 +354,8 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         "SessionPersistence", uselist=False, cascade="delete",
         back_populates="pool")
     _default_listeners = orm.relationship("Listener", uselist=True,
-                                          back_populates="default_pool")
+                                          back_populates="default_pool",
+                                          cascade_backrefs=False)
     l7policies = orm.relationship("L7Policy", uselist=True,
                                   back_populates="redirect_pool")
     _tags = orm.relationship(
@@ -430,6 +432,9 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
     provider = sa.Column(sa.String(64), nullable=True)
     vip = orm.relationship('Vip', cascade='delete', uselist=False,
                            backref=orm.backref('load_balancer', uselist=False))
+    additional_vips = orm.relationship(
+        'AdditionalVip', cascade='delete', uselist=True,
+        backref=orm.backref('load_balancer', uselist=False))
     pools = orm.relationship('Pool', cascade='delete', uselist=True,
                              back_populates="load_balancer")
     listeners = orm.relationship('Listener', cascade='delete', uselist=True,
@@ -450,6 +455,7 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
         sa.ForeignKey("availability_zone.name",
                       name="fk_load_balancer_availability_zone_name"),
         nullable=True)
+    flavor: Mapped["Flavor"] = orm.relationship("Flavor")
 
     def __str__(self):
         return (f"LoadBalancer(id={self.id!r}, name={self.name!r}, "
@@ -502,6 +508,28 @@ class Vip(base_models.BASE):
     octavia_owned = sa.Column(sa.Boolean(), nullable=True)
 
 
+class AdditionalVip(base_models.BASE):
+
+    __data_model__ = data_models.AdditionalVip
+
+    __tablename__ = "additional_vip"
+
+    __table_args__ = (
+        sa.PrimaryKeyConstraint('load_balancer_id', 'subnet_id',
+                                name='pk_add_vip_load_balancer_subnet'),
+    )
+
+    load_balancer_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("load_balancer.id",
+                      name="fk_add_vip_load_balancer_id"),
+        nullable=False, index=True)
+    ip_address = sa.Column(sa.String(64), nullable=True)
+    port_id = sa.Column(sa.String(36), nullable=True)
+    subnet_id = sa.Column(sa.String(36), nullable=True)
+    network_id = sa.Column(sa.String(36), nullable=True)
+
+
 class Listener(base_models.BASE, base_models.IdMixin,
                base_models.ProjectMixin, models.TimestampMixin,
                base_models.NameMixin, base_models.TagMixin):
@@ -548,7 +576,8 @@ class Listener(base_models.BASE, base_models.IdMixin,
     load_balancer = orm.relationship("LoadBalancer", uselist=False,
                                      back_populates="listeners")
     default_pool = orm.relationship("Pool", uselist=False,
-                                    back_populates="_default_listeners")
+                                    back_populates="_default_listeners",
+                                    cascade_backrefs=False)
     sni_containers = orm.relationship(
         'SNI', cascade='all,delete-orphan',
         uselist=True, backref=orm.backref('listener', uselist=False))
@@ -574,6 +603,9 @@ class Listener(base_models.BASE, base_models.IdMixin,
     tls_ciphers = sa.Column(sa.String(2048), nullable=True)
     tls_versions = sa.Column(ScalarListType(), nullable=True)
     alpn_protocols = sa.Column(ScalarListType(), nullable=True)
+    hsts_max_age = sa.Column(sa.Integer, nullable=True)
+    hsts_include_subdomains = sa.Column(sa.Boolean, nullable=True)
+    hsts_preload = sa.Column(sa.Boolean, nullable=True)
 
     _tags = orm.relationship(
         'Tags',
@@ -882,6 +914,7 @@ class Flavor(base_models.BASE,
         sa.ForeignKey("flavor_profile.id",
                       name="fk_flavor_flavor_profile_id"),
         nullable=False)
+    flavor_profile: Mapped["FlavorProfile"] = orm.relationship("FlavorProfile")
 
 
 class AvailabilityZoneProfile(base_models.BASE, base_models.IdMixin,
@@ -917,6 +950,8 @@ class AvailabilityZone(base_models.BASE,
         sa.ForeignKey("availability_zone_profile.id",
                       name="fk_az_az_profile_id"),
         nullable=False)
+    availability_zone_profile: Mapped["AvailabilityZoneProfile"] = (
+        orm.relationship("AvailabilityZoneProfile"))
 
 
 class ClientAuthenticationMode(base_models.BASE):
