@@ -100,7 +100,8 @@ class AbstractNetworkDriver(object, metaclass=abc.ABCMeta):
         balancer.
 
         :param load_balancer: octavia.common.data_models.LoadBalancer instance
-        :return: octavia.common.data_models.VIP
+        :return: octavia.common.data_models.Vip,
+                 list(octavia.common.data_models.AdditionalVip)
         :raises: AllocateVIPException, PortNotFound, SubnetNotFound
         """
 
@@ -353,20 +354,6 @@ class AbstractNetworkDriver(object, metaclass=abc.ABCMeta):
         :raises: NotFound, NetworkNotFound, SubnetNotFound, PortNotFound
         """
 
-    @abc.abstractmethod
-    def wait_for_port_detach(self, amphora):
-        """Waits for the amphora ports device_id to be unset.
-
-        This method waits for the ports on an amphora device_id
-        parameter to be '' or None which signifies that nova has
-        finished detaching the port from the instance.
-
-        :param amphora: Amphora to wait for ports to detach.
-        :returns: None
-        :raises TimeoutException: Port did not detach in interval.
-        :raises PortNotFound: Port was not found by neutron.
-        """
-
     @abc.abstractmethod
     def update_vip_sg(self, load_balancer, vip):
         """Updates the security group for a VIP
