@@ -12,7 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
@@ -21,6 +22,7 @@ import pecan.testing
 
 from octavia.api import config as pconfig
 from octavia.common import constants
+from octavia.common import exceptions
 from octavia.db import api as db_api
 from octavia.db import repositories
 from octavia.tests.functional.db import base as base_db_test
@@ -28,21 +30,38 @@ from octavia.tests.functional.db import base as base_db_test
 
 class BaseAPITest(base_db_test.OctaviaDBTestBase):
 
-    BASE_PATH = '/v2.0/lbaas'
+    BASE_PATH = '/v2'
+    BASE_PATH_v2_0 = '/v2.0'
+
+    # /lbaas/flavors
+    FLAVORS_PATH = '/flavors'
+    FLAVOR_PATH = FLAVORS_PATH + '/{flavor_id}'
+
+    # /lbaas/flavorprofiles
+    FPS_PATH = '/flavorprofiles'
+    FP_PATH = FPS_PATH + '/{fp_id}'
+
+    # /lbaas/availabilityzones
+    AZS_PATH = '/availabilityzones'
+    AZ_PATH = AZS_PATH + '/{az_name}'
+
+    # /lbaas/availabilityzoneprofiles
+    AZPS_PATH = '/availabilityzoneprofiles'
+    AZP_PATH = AZPS_PATH + '/{azp_id}'
 
     # /lbaas/loadbalancers
-    LBS_PATH = '/loadbalancers'
+    LBS_PATH = '/lbaas/loadbalancers'
     LB_PATH = LBS_PATH + '/{lb_id}'
     LB_STATUS_PATH = LB_PATH + '/statuses'
     LB_STATS_PATH = LB_PATH + '/stats'
 
     # /lbaas/listeners/
-    LISTENERS_PATH = '/listeners'
+    LISTENERS_PATH = '/lbaas/listeners'
     LISTENER_PATH = LISTENERS_PATH + '/{listener_id}'
     LISTENER_STATS_PATH = LISTENER_PATH + '/stats'
 
     # /lbaas/pools
-    POOLS_PATH = '/pools'
+    POOLS_PATH = '/lbaas/pools'
     POOL_PATH = POOLS_PATH + '/{pool_id}'
 
     # /lbaas/pools/{pool_id}/members
@@ -50,30 +69,54 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
     MEMBER_PATH = MEMBERS_PATH + '/{member_id}'
 
     # /lbaas/healthmonitors
-    HMS_PATH = '/healthmonitors'
+    HMS_PATH = '/lbaas/healthmonitors'
     HM_PATH = HMS_PATH + '/{healthmonitor_id}'
 
     # /lbaas/l7policies
-    L7POLICIES_PATH = '/l7policies'
+    L7POLICIES_PATH = '/lbaas/l7policies'
     L7POLICY_PATH = L7POLICIES_PATH + '/{l7policy_id}'
     L7RULES_PATH = L7POLICY_PATH + '/rules'
     L7RULE_PATH = L7RULES_PATH + '/{l7rule_id}'
 
-    QUOTAS_PATH = '/quotas'
+    QUOTAS_PATH = '/lbaas/quotas'
     QUOTA_PATH = QUOTAS_PATH + '/{project_id}'
     QUOTA_DEFAULT_PATH = QUOTAS_PATH + '/{project_id}/default'
 
+    AMPHORAE_PATH = '/octavia/amphorae'
+    AMPHORA_PATH = AMPHORAE_PATH + '/{amphora_id}'
+    AMPHORA_FAILOVER_PATH = AMPHORA_PATH + '/failover'
+    AMPHORA_STATS_PATH = AMPHORA_PATH + '/stats'
+    AMPHORA_CONFIG_PATH = AMPHORA_PATH + '/config'
+
+    PROVIDERS_PATH = '/lbaas/providers'
+    FLAVOR_CAPABILITIES_PATH = (
+        PROVIDERS_PATH + '/{provider}/flavor_capabilities')
+    AVAILABILITY_ZONE_CAPABILITIES_PATH = (
+        PROVIDERS_PATH + '/{provider}/availability_zone_capabilities')
+
     NOT_AUTHORIZED_BODY = {
         'debuginfo': None, 'faultcode': 'Client',
         'faultstring': 'Policy does not allow this request to be performed.'}
 
     def setUp(self):
-        super(BaseAPITest, self).setUp()
+        super().setUp()
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
-        self.conf.config(group='api_settings', api_handler='simulated_handler')
+        # Mock log_opt_values, it prevents the dump of the configuration
+        # with LOG.info for each test. It saves a lot of time when running
+        # the functional tests.
+        self.conf.conf.log_opt_values = mock.MagicMock()
         self.conf.config(group="controller_worker",
                          network_driver='network_noop_driver')
         self.conf.config(group='api_settings', auth_strategy=constants.NOAUTH)
+        self.conf.config(group='api_settings',
+                         default_provider_driver='noop_driver')
+        # We still need to test with the "octavia" alias
+        self.conf.config(group='api_settings',
+                         enabled_provider_drivers={
+                             'amphora': 'Amp driver.',
+                             'noop_driver': 'NoOp driver.',
+                             'noop_driver-alt': 'NoOp driver alt alisas.',
+                             'octavia': 'Octavia driver.'})
         self.lb_repo = repositories.LoadBalancerRepository()
         self.listener_repo = repositories.ListenerRepository()
         self.listener_stats_repo = repositories.ListenerStatisticsRepository()
@@ -83,14 +126,15 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
         self.l7rule_repo = repositories.L7RuleRepository()
         self.health_monitor_repo = repositories.HealthMonitorRepository()
         self.amphora_repo = repositories.AmphoraRepository()
-        patcher = mock.patch('octavia.api.handlers.controller_simulator.'
-                             'handler.SimulatedControllerHandler')
-        self.handler_mock = patcher.start()
+        self.flavor_repo = repositories.FlavorRepository()
+        self.flavor_profile_repo = repositories.FlavorProfileRepository()
+        patcher2 = mock.patch('octavia.certificates.manager.barbican.'
+                              'BarbicanCertManager')
+        self.cert_manager_mock = patcher2.start()
         self.app = self._make_app()
         self.project_id = uuidutils.generate_uuid()
 
         def reset_pecan():
-            patcher.stop()
             pecan.set_config({}, overwrite=True)
 
         self.addCleanup(reset_pecan)
@@ -105,12 +149,17 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
         self.addCleanup(check_quota_met_true_mock.stop)
 
     def _make_app(self):
+        # Note: we need to set argv=() to stop the wsgi setup_app from
+        # pulling in the testing tool sys.argv
         return pecan.testing.load_test_app({'app': pconfig.app,
-                                            'wsme': pconfig.wsme})
+                                            'wsme': pconfig.wsme}, argv=())
 
     def _get_full_path(self, path):
         return ''.join([self.BASE_PATH, path])
 
+    def _get_full_path_v2_0(self, path):
+        return ''.join([self.BASE_PATH_v2_0, path])
+
     def _build_body(self, json):
         return {self.root_tag: json}
 
@@ -131,9 +180,13 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                                    expect_errors=expect_errors)
         return response
 
-    def post(self, path, body, headers=None, status=201, expect_errors=False):
+    def post(self, path, body, headers=None, status=201, expect_errors=False,
+             use_v2_0=False):
         headers = headers or {}
-        full_path = self._get_full_path(path)
+        if use_v2_0:
+            full_path = self._get_full_path_v2_0(path)
+        else:
+            full_path = self._get_full_path(path)
         response = self.app.post_json(full_path,
                                       params=body,
                                       headers=headers,
@@ -161,6 +214,39 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                                 expect_errors=expect_errors)
         return response
 
+    def create_flavor(self, name, description, flavor_profile_id, enabled):
+        req_dict = {'name': name, 'description': description,
+                    'flavor_profile_id': flavor_profile_id,
+                    'enabled': enabled}
+        body = {'flavor': req_dict}
+        response = self.post(self.FLAVORS_PATH, body)
+        return response.json.get('flavor')
+
+    def create_flavor_profile(self, name, provider_name, flavor_data):
+        req_dict = {'name': name, 'provider_name': provider_name,
+                    constants.FLAVOR_DATA: flavor_data}
+        body = {'flavorprofile': req_dict}
+        response = self.post(self.FPS_PATH, body)
+        return response.json.get('flavorprofile')
+
+    def create_availability_zone(self, name, description,
+                                 availability_zone_profile_id, enabled):
+        req_dict = {
+            'name': name, 'description': description,
+            'availability_zone_profile_id': availability_zone_profile_id,
+            'enabled': enabled}
+        body = {'availability_zone': req_dict}
+        response = self.post(self.AZS_PATH, body)
+        return response.json.get('availability_zone')
+
+    def create_availability_zone_profile(self, name, provider_name,
+                                         availability_zone_data):
+        req_dict = {'name': name, 'provider_name': provider_name,
+                    constants.AVAILABILITY_ZONE_DATA: availability_zone_data}
+        body = {'availability_zone_profile': req_dict}
+        response = self.post(self.AZPS_PATH, body)
+        return response.json.get('availability_zone_profile')
+
     def create_load_balancer(self, vip_subnet_id,
                              **optionals):
         req_dict = {'vip_subnet_id': vip_subnet_id,
@@ -183,7 +269,7 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
 
     def create_listener_stats(self, listener_id, amphora_id):
         db_ls = self.listener_stats_repo.create(
-            db_api.get_session(), listener_id=listener_id,
+            self.session, listener_id=listener_id,
             amphora_id=amphora_id, bytes_in=0,
             bytes_out=0, active_connections=0, total_connections=0,
             request_errors=0)
@@ -194,7 +280,7 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                                       active_connections=0,
                                       total_connections=0, request_errors=0):
         db_ls = self.listener_stats_repo.create(
-            db_api.get_session(), listener_id=listener_id,
+            self.session, listener_id=listener_id,
             amphora_id=amphora_id, bytes_in=bytes_in,
             bytes_out=bytes_out, active_connections=active_connections,
             total_connections=total_connections, request_errors=request_errors)
@@ -297,6 +383,9 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
         response = self.put(path, body, status=202)
         return response.json
 
+    # NOTE: This method should be used cautiously. On load balancers with a
+    # significant amount of children resources, it will update the status for
+    # each and every resource and thus taking a lot of DB time.
     def _set_lb_and_children_statuses(self, lb_id, prov_status, op_status,
                                       autodetect=True):
         self.set_object_status(self.lb_repo, lb_id,
@@ -366,6 +455,9 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                                        provisioning_status=hm_prov,
                                        operating_status=op_status)
 
+    # NOTE: This method should be used cautiously. On load balancers with a
+    # significant amount of children resources, it will update the status for
+    # each and every resource and thus taking a lot of DB time.
     def set_lb_status(self, lb_id, status=None):
         explicit_status = True if status is not None else False
         if not explicit_status:
@@ -380,25 +472,29 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
             op_status = db_lb.operating_status
         self._set_lb_and_children_statuses(lb_id, status, op_status,
                                            autodetect=not explicit_status)
-        return self.get(self.LB_PATH.format(lb_id=lb_id)).json
+        if status != constants.DELETED:
+            return self.get(self.LB_PATH.format(lb_id=lb_id)).json
 
     @staticmethod
     def set_object_status(repo, id_, provisioning_status=constants.ACTIVE,
                           operating_status=constants.ONLINE):
-        repo.update(db_api.get_session(), id_,
+        session = db_api.get_session()
+        repo.update(session, id_,
                     provisioning_status=provisioning_status,
                     operating_status=operating_status)
+        session.commit()
 
     def assert_final_listener_statuses(self, lb_id, listener_id, delete=False):
         expected_prov_status = constants.ACTIVE
         expected_op_status = constants.ONLINE
-        if delete:
-            expected_prov_status = constants.DELETED
-            expected_op_status = constants.OFFLINE
         self.set_lb_status(lb_id, status=expected_prov_status)
-        self.assert_correct_listener_status(expected_prov_status,
-                                            expected_op_status,
-                                            listener_id)
+        try:
+            self.assert_correct_listener_status(expected_prov_status,
+                                                expected_op_status,
+                                                listener_id)
+        except exceptions.NotFound:
+            if not delete:
+                raise
 
     def assert_correct_lb_status(self, lb_id,
                                  operating_status, provisioning_status):
