@@ -78,10 +78,10 @@ class TestBarbicanManager(base.TestCase):
 
         self.empty_container = mock.Mock(spec=containers.CertificateContainer)
 
-        self.secret1 = mock.Mock(spec=secrets.Secret)
-        self.secret2 = mock.Mock(spec=secrets.Secret)
-        self.secret3 = mock.Mock(spec=secrets.Secret)
-        self.secret4 = mock.Mock(spec=secrets.Secret)
+        self.secret1 = mock.Mock()
+        self.secret2 = mock.Mock()
+        self.secret3 = mock.Mock()
+        self.secret4 = mock.Mock()
 
         # Mock out the client
         self.bc = mock.Mock()
@@ -95,7 +95,7 @@ class TestBarbicanManager(base.TestCase):
         self.context = mock.Mock()
         self.context.project_id = PROJECT_ID
 
-        super(TestBarbicanManager, self).setUp()
+        super().setUp()
 
     def test_store_cert(self):
         # Mock out the client
@@ -238,20 +238,21 @@ class TestBarbicanManager(base.TestCase):
 
     def test_get_cert_no_registration_raise_on_secret_access_failure(self):
         self.bc.containers.get.return_value = self.container
-        type(self.certificate).payload = mock.PropertyMock(
-            side_effect=ValueError)
+        with mock.patch('barbicanclient.v1.secrets.Secret.payload',
+                        new_callable=mock.PropertyMock) as mock_payload:
+            mock_payload.side_effect = ValueError
 
-        # Get the container data
-        self.assertRaises(
-            ValueError, self.cert_manager.get_cert,
-            context=self.context,
-            cert_ref=self.container_ref, check_only=True
-        )
+            # Get the container data
+            self.assertRaises(
+                ValueError, self.cert_manager.get_cert,
+                context=self.context,
+                cert_ref=self.container_ref, check_only=True
+            )
 
-        # 'get' should be called once with the container_ref
-        self.bc.containers.get.assert_called_once_with(
-            container_ref=self.container_ref
-        )
+            # 'get' should be called once with the container_ref
+            self.bc.containers.get.assert_called_once_with(
+                container_ref=self.container_ref
+            )
 
     def test_delete_cert(self):
         # Attempt to deregister as a consumer
