@@ -586,7 +586,8 @@ RET_SCTP_LISTENER = {
 
 
 def sample_listener_loadbalancer_tuple(
-        topology=None, enabled=True, pools=None, additional_vips=False):
+        topology=None, enabled=True, vip=None, pools=None,
+        additional_vips=False):
     if topology and topology in ['ACTIVE_STANDBY', 'ACTIVE_ACTIVE']:
         more_amp = True
     else:
@@ -598,7 +599,7 @@ def sample_listener_loadbalancer_tuple(
     return in_lb(
         id='sample_loadbalancer_id_1',
         name='test-lb',
-        vip=sample_vip_tuple(),
+        vip=vip or sample_vip_tuple(),
         amphorae=[sample_amphora_tuple(role=constants.ROLE_MASTER),
                   sample_amphora_tuple(
                       id='sample_amphora_id_2',
@@ -618,13 +619,13 @@ def sample_listener_loadbalancer_tuple(
 
 
 def sample_lb_with_udp_listener_tuple(
-        topology=None, enabled=True, pools=None):
+        topology=None, enabled=True, listeners=None, pools=None):
     if topology and topology in ['ACTIVE_STANDBY', 'ACTIVE_ACTIVE']:
         more_amp = True
     else:
         more_amp = False
         topology = constants.TOPOLOGY_SINGLE
-    listeners = [sample_listener_tuple(
+    listeners = listeners or [sample_listener_tuple(
         proto=constants.PROTOCOL_UDP,
         persistence_type=constants.SESSION_PERSISTENCE_SOURCE_IP,
         persistence_timeout=33,
@@ -706,7 +707,9 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                           backend_alpn_protocols=constants.
                           AMPHORA_SUPPORTED_ALPN_PROTOCOLS,
                           include_pools=True,
-                          additional_vips=False):
+                          additional_vips=False,
+                          hsts_max_age=10_000_000,
+                          hsts_include_subdomains=True, hsts_preload=True):
     proto = 'HTTP' if proto is None else proto
     if be_proto is None:
         be_proto = 'HTTP' if proto == 'TERMINATED_HTTPS' else proto
@@ -730,7 +733,9 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                     'timeout_tcp_inspect, client_ca_tls_certificate_id, '
                     'client_ca_tls_certificate, client_authentication, '
                     'client_crl_container_id, provisioning_status, '
-                    'tls_ciphers, tls_versions, alpn_protocols')
+                    'tls_ciphers, tls_versions, alpn_protocols, '
+                    'hsts_max_age, hsts_include_subdomains, hsts_preload'
+    )
     if l7:
         pools = [
             sample_pool_tuple(
@@ -858,7 +863,10 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
         provisioning_status=provisioning_status,
         tls_ciphers=tls_ciphers,
         tls_versions=tls_versions,
-        alpn_protocols=alpn_protocols
+        alpn_protocols=alpn_protocols,
+        hsts_max_age=hsts_max_age,
+        hsts_include_subdomains=hsts_include_subdomains,
+        hsts_preload=hsts_preload,
     )
     if recursive_nest:
         listener.load_balancer.listeners.append(listener)
